	.section .text1,"ax",%progbits
	.space 0x10
	.section .text2,"ax",%progbits
	.space 0x20
	.section .silly-name1,"ax",%progbits
	.space 0x10
	.section .silly-name2,"ax",%progbits
	.space 0x20
	.text
	.space 0x30
